@@ -1,3 +1,4 @@
+import pytest
 import numpy as np
 from scipy.optimize import minimize, Bounds
 
@@ -13,6 +14,7 @@ def test_gh10880():
     minimize(lambda x: x**2, x0=2., method='trust-constr',
              bounds=bnds, options=opts)
 
+@pytest.mark.xslow
 def test_gh12922():
     # checks that verbose reporting works with trust-constr for
     # general constraints
@@ -24,9 +26,9 @@ def test_gh12922():
     x0 = np.linspace(-5, 5, n)
 
     opts = {'maxiter': 1000, 'verbose': 2}
-    result = minimize(objective, x0=x0, method='trust-constr',
+    minimize(objective, x0=x0, method='trust-constr',
                       constraints=cons, options=opts)
 
     opts = {'maxiter': 1000, 'verbose': 3}
-    result = minimize(objective, x0=x0, method='trust-constr',
+    minimize(objective, x0=x0, method='trust-constr',
                       constraints=cons, options=opts)
